#!/bin/ksh

# IBM Confidential
# OCO Source Material
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2010
# The source code for this program is not published or otherwise divested of
# its trade secret, irrespective of what has been deposited with the U.S.
# Copyright Office.

helpmsg()
{
  echo "Usage:  $0 [ <path_to_directory> | -h | -H ]"
  echo ""
  echo "  path_to_directory - Directory name with path contains headers of IBM DB2 logsets for processing."
  echo "                      Current directory is used if parameter is not specified."
  echo "  -h|-H  - Screen with this help."
  echo ""

  exit 0
}

echo ""
[ "$1" == "-h" -o "$1" == "-H" ] && helpmsg

CURR_DIR="$(pwd)"
if [ -z "$1" ]; then
   WORK_DIR="${CURR_DIR}"
   echo "Path to logset directory is not specified. Using default value - current directory."
   echo ""
else
   WORK_DIR="$1"
fi

cd "${WORK_DIR}"
if [ $? -ne 0 ]; then
   echo "Error: wrong parameter."
   echo ""
   helpmsg
fi

WORK_DIR="$(pwd)"
cd "${CURR_DIR}"

TEMP_FILE="db2udb_logset_header.tmp"
LAST_CHAR=$(echo ${WORK_DIR} | sed -n 's/.*\(.\)$/\1/p')
if [ "${LAST_CHAR}" != "/" ]; then
   WORK_DIR="${WORK_DIR}/"
fi

echo "Looking for logset header files of 'IBM DB2 UDB' or 'IBM DB2 8.1 - 9.1 SP3' platform type for processing."
echo "Working directory - '${WORK_DIR}':"
echo ""

HEADER_NUMBER=0
PROC_NUMBER=0
ERROR_NUMBER=0

for HEADER_FILE in $(ls -1 "${WORK_DIR}"*0 2>/dev/null); do
    ((HEADER_NUMBER+=1))
    TYPE=$(sed -n 's/^((string) "PlatformType") = (((string) "\(.*\)"))/\1/p' "${HEADER_FILE}")
    if echo "${TYPE}" | grep -q "^IBM DB2 UDB" || \
       echo "${TYPE}" | grep -q "^IBM DB2 8\.1 - 9\.1 SP3"; then
       ((PROC_NUMBER+=1))
       grep -v "^((string) \"encoding[1-6]\") = (((string) \"UTF-8\"))" "${HEADER_FILE}" > "${TEMP_FILE}"
       mv -f "${TEMP_FILE}" "${HEADER_FILE}" >/dev/null
       if [ $? -eq 0 ]; then
          echo  "'${HEADER_FILE}' file processed."
       else
          ((ERROR_NUMBER+=1))
          echo "Failure processing file '${HEADER_FILE}'."
       fi
    fi
done

[ ${HEADER_NUMBER} -gt 0 ] && echo ""
echo "Total number of found header files: ${HEADER_NUMBER}"
echo "Number of processed specified files: ${PROC_NUMBER}"
echo "Number of files, processed with errors: ${ERROR_NUMBER}"
echo ""
echo "Finished."

exit 0
