###########################################################BeginFile############
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# File-stamp: <010629 arthur 14:51 genact-etrust>
#
# Name:
#   genact-etrust
#
# Purpose:
#   To collect log entries that were added since
#   the last run and write them to file $1
#   
# Notes:
#   This script should be called from genact-main, since it depends on it to
#   set things up.
#   
# History:
#   010629 0.0.31 AWH: Integrated log routing daemon based actuator script. The
#                      previous seaudit based one has been replaced.
#   01.02.2005 VL: WE0407002 - fixed.
###########################################################EndFile##############

# Source our function library. It contains generic functions.
. $CEA_GENACT_FUNCLIB || exit 1

# We expect the name of the outfile in $1 (not checked; we trust genact-main).
OUTFILE=$1

# Define cleanup function (which is called by 'die'). This one just removes
# $OUTFILE when we're exiting unsuccessfully.
cleanup ()
{
    [[ $1 -gt 0 ]] && rm -f $OUTFILE;
}

REFORMAT_CMD=../bin/serefmt

testrights $REFORMAT_CMD "X"

# In $OPTION2 We expect the location of the audit log as written by the eTrust
# Access Control log routing emitter daemon.
[[ -n "$OPTION2" ]] || die 2 "Missing event source property 'Option 2'; it should contain the name and path of the log file to collect, as written by the eTrust Access Control log routing emitter daemon.";
LOG_SOURCE=$OPTION2

# Option 3 may contain the name of a directory to archive the collected audit
# data to. If this option is not given, collected data will be removed from
# the audited system.
if [[ -n "$OPTION3" ]]
then
  LOG_COPY=$OPTION3/audit.trail.`date +%m%d%Y%H%M%S`
  ARCHIVE=1
else
  LOG_COPY=$OPTION2.copy.`date +%m%d%Y%H%M%S`
  ARCHIVE=0
fi

# Does the eTrust Log file exist?
if [[ ! -f $LOG_SOURCE ]]
then
    error "$LOG_SOURCE not found. Collect aborted."
    die 3
fi

debug "Moving $LOG_SOURCE to $LOG_COPY"
mv $LOG_SOURCE $LOG_COPY || die 3 "Error moving $LOG_SOURCE to $LOG_COPY. Maybe the archive directory does not exist or is on a different partition than the collected log file."

debug "Calling $REFORMAT_CMD $LOG_COPY $OUTFILE"
$REFORMAT_CMD $LOG_COPY $OUTFILE || die 4 "Error calling $REFORMAT_CMD $LOG_COPY $OUTFILE. WARNING: the data in $LOG_COPY will never be collected!"

# If data should not be archived locally, remove the copy of the collected
# log file.
if [[ $ARCHIVE -eq 0 ]]
then
    debug "Removing $LOG_COPY."
    rm -f $LOG_COPY
fi

# Does the eTrust Log file exist?
if [[ ! -f $OUTFILE ]]
then
    error "$OUTFILE not found. Collect aborted."
    die 3
fi

# Clean up and exit successfully.
die 0

