#!/bin/sh
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
###############################################################################
#
# Name:
#   getaselogs.sh
#
# Purpose:
#   Actuator script for Sybase ASE ES for Unix platforms.
#
# History:
#   2006dec07 ScnSoft Pipe-A  EE0609007: Created.
#   071212  0.0.27 2944109017: Removed carriage return symbols. 
###############################################################################

our_exit()
{
  debug $2 
  finish_log_in_error
  exit $1
}
 
finish_log_in_error()
{
  rm -f $TEMPFILE1
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
}

create_view()
{
  RETURN_VALUE=`${OCS_HOME}/bin/isql -S${ASE_INSTANCE} -U${ASE_USER} -P${ASE_PWD} -Dtempdb -b -n -w1024 << ! 2>> $CEA_GENACT_DEBUG_LOG
    set flushmessage on
    set nocount on
    set statistics io, subquerycache, time off

    if exists (select * from sysobjects where name = "${VIEW_NAME}")
    begin
      drop view aselogsview
    end

    declare @table_count int
    select @table_count = count(*) from sybsecurity..sysobjects where name like "sysaudits[_]0[1-8]"

    declare @query varchar(1500)
    select @query = 'create view ${VIEW_NAME} as'
    while @table_count > 1
    begin
      select @query = @query + ' select sybsecurity..sysaudits_0' + convert(char(1), @table_count) + '.* from sybsecurity..sysaudits_0'
      select @query = @query + convert(char(1), @table_count)
      select @query = @query + ' union all'
      select @table_count = @table_count - 1
    end

    if @table_count = 1
    begin
      select @query = @query + ' select * from sybsecurity..sysaudits_01'
      exec(@query)
      select '.'
    end
go
!`
  [[ $? -eq 0 ]] || our_exit 2 "Error occured: ${RETURN_VALUE}."
  RETURN_VALUE2=`echo ${RETURN_VALUE}|grep "Msg"`
  [[ -z ${RETURN_VALUE2} ]] || our_exit 2 "Error occured: ${RETURN_VALUE}."
}

get_max_time()
{
  LAST_TIME_NOW=`${OCS_HOME}/bin/isql -S${ASE_INSTANCE} -U${ASE_USER} -P${ASE_PWD} -Dtempdb -b -n << ! 2>> $CEA_GENACT_DEBUG_LOG
    set flushmessage on
    set nocount on
    set statistics io, subquerycache, time off
    
    declare @max_time datetime
    select @max_time = max(eventtime) from ${VIEW_NAME}
    if( @max_time != null ) select convert(char(26), @max_time, 109)
go
!`
  [[ $? -eq 0 ]] || our_exit 2 "Error occured: ${LAST_TIME_NOW}."
}

store_initial_time()
{
  LAST_TIME_NOW=`${OCS_HOME}/bin/isql -S${ASE_INSTANCE} -U${ASE_USER} -P${ASE_PWD} -b -n << ! 2>> $CEA_GENACT_DEBUG_LOG
    set flushmessage on
    set nocount on
    set statistics io, subquerycache, time off
    select convert(char(26), dateadd(day, -2, getdate()), 109)
go
!`
  [[ $? -eq 0 ]] || our_exit 2 "Error occured: ${LAST_TIME_NOW}."
  echo "LAST_TIME_BEFORE='${LAST_TIME_NOW}'">${PROPSFILE}
}

get_records()
{
  RETURN_VALUE=`${OCS_HOME}/bin/isql -S${ASE_INSTANCE} -U${ASE_USER} -P${ASE_PWD} -Dtempdb -b -n -w1024 -s"
"<< ! > ${TEMPFILE1}0 2>> $CEA_GENACT_DEBUG_LOG
    set flushmessage on
    set nocount on
    set statistics io, subquerycache, time off

    select "SYBASE" + char(9) +
      convert(varchar(16), event) + char(9) + 
      convert(varchar(16), eventmod) + char(9) +
      convert(varchar(16), spid) + char(9) +
      str(datepart(yy, eventtime), 4) + '-' + 
        str_replace(str(datepart(mm, eventtime), 2) + '-' + str(datepart(dd, eventtime), 2), ' ', '0') + ' ' +
        str_replace(str(datepart(hh, eventtime), 2) + ':' + str(datepart(mi, eventtime), 2) + ':' +
          str(datepart(ss, eventtime), 2) + '.' + str(datepart(ms, eventtime), 3), ' ', '0') + char(9) +
      convert(varchar(16), sequence) + char(9) +
      convert(varchar(16), suid) + char(9) +
      convert(varchar(16), dbid) + char(9) +
      convert(varchar(16), objid) + char(9) +
      convert(char(6), inttohex(substring(xactid, 1, 3))) + convert(char(6), inttohex(substring(xactid, 4, 6))) + char(9) + 
      str_replace( 
        str_replace(loginname, char(9), ' ') + char(9) + 
        str_replace(dbname, char(9), ' ') + char(9) + 
        str_replace(objname, char(9), ' ') + char(9) + 
        str_replace(objowner, char(9), ' ') + char(9) + 
        str_replace(extrainfo, char(9), ' ') + char(9),
        char(10)+char(13), '  '),
      eventtime, sequence
    from ${VIEW_NAME}
    where ${COND1} and eventtime < convert(datetime, '${LAST_TIME_NOW}', 109) 
    order by eventtime, sequence
go
!`
  [[ $? -eq 0 ]] || our_exit 2 "Error occured: ${RETURN_VALUE}."
  
  echo "#Fields: SYBASE	event	eventmod	spid	eventtime	sequence	suid	dbid	objid	xactid	loginname	dbname	objname	objowner	extrainfo	">${TEMPFILE1}

  # remove the trailing spaces from the spool buffer
  grep "SYBASE" ${TEMPFILE1}0 | sed -e 's/ *$//'>>${TEMPFILE1} || our_exit 3 "Internal error."
  rm ${TEMPFILE1}0 
}

# Source our function library. It contains generic functions. 
. $CEA_GENACT_FUNCLIB || exit 1
 
CEA_GENACT_DEBUG_LOG=../log/${PROPSFILE}.log
debug_init

[[ -n "${OPTION2}" ]] \
  || our_exit 1 "Option #2 'Sybase Instance' must be specified!"

[[ -n "${OPTION3}" ]] \
  || our_exit 1 "Option #3 'Sybase User Name' must be specified!"

[[ -n "${OPTION5}" ]] \
  || our_exit 1 "Option #5 'Sybase Open Client/Server Folder' must be specified!"

[[ -n "${PROPSFILE}" ]] \
  || our_exit 1 "Properties file must be specified!"

# Parameters needed for sybase access
ASE_USER=${OPTION3}
ASE_PWD=$(../bin/cmtool -d "${OPTION4}")
ASE_INSTANCE=${OPTION2}
OCS_HOME=${OPTION5}

# Check if ${OCS_HOME}/bin/isql is existing.
debug "Checking if ${OCS_HOME}/bin/isql is existing."
if [[ ! -s ${OCS_HOME}/bin/isql ]]
then
  our_exit 2 "Error occured: ${OCS_HOME}/bin/isql not found."
fi

test -r ${OCS_HOME}/bin/isql || our_exit 2 "${OCS_HOME}/bin/isql is not accessible for reading/executing."
test -x ${OCS_HOME}/bin/isql || our_exit 2 "${OCS_HOME}/bin/isql is not accessible for reading/executing."

# Obtain timestamp value from properties file.
if [[ -s ${PROPSFILE} ]]
then 
   LAST_TIME_BEFORE=`sed -n "s/^LAST_TIME_BEFORE=\'\(.*\)\'/\1/p" $PROPSFILE` || \
     our_exit 3 "Error reading actuator state file ${PROPSFILE}"
fi

INITIAL_COLLECT=0
if [ -z "${LAST_TIME_BEFORE}" ]
then
  debug "Initial collect."
  INITIAL_COLLECT=1
  COND1=`echo "eventtime >= dateadd(day, -2, getdate())"`
else
  debug "Non-initial collect, start timestamp: '${LAST_TIME_BEFORE}'."
  COND1=`echo "eventtime >= convert(datetime, '${LAST_TIME_BEFORE}', 109)"`
fi

# Set $SYBASE environment variable.
SYBASE=${OCS_HOME%/*}
export SYBASE

# Create view that combines all records of all audit tables
VIEW_NAME="aselogsview"
create_view

# Obtain the greatest timestamp among all audit records.
debug "Obtain the greatest timestamp among all audit records..."
get_max_time
if [ -z "${LAST_TIME_NOW}" ]
then
  debug "The greatest timestamp is empty! There are no relevant records."
  [[ ${INITIAL_COLLECT} -eq 0 ]] || store_initial_time
  echo "#Fields: SYBASE	event	eventmod	spid	eventtime	sequence	suid	dbid	objid	xactid	loginname	dbname	objname	objowner	extrainfo	">${TEMPFILE1}
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
  exit 0
fi
debug "The greatest timestamp: ${LAST_TIME_NOW}."

# Obtain actual audit records.
debug "Obtain actual audit records..."
get_records

echo "LAST_TIME_BEFORE='${LAST_TIME_NOW}'">${PROPSFILE}
echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`), `date`." >> $CEA_GENACT_DEBUG_LOG
