#!/bin/ksh

# input parameters
TEMP_FILE=$1
LDAPHOST=""
LDAPPORT=$3
USERNAME=$4
PASSWORD=$5
PROPS_FILE=$6

act_log_file="../log/`basename $PROPS_FILE`.log";
touch $act_log_file 

echo "[`date`] Processing started">$act_log_file;

if [ -z "$TSIEM_HOME" ];
then
   echo "[`date`] Collect from non - TSIEM Server system" >>$act_log_file;
   echo "[`date`] Path to the JRE binaries must be available in the PATH variable for success collect">>$act_log_file;
else
   echo "[`date`] Collect from TSIEM Server system">>$act_log_file;
   echo "[`date`] Adding TSIEM JRE binaries path to the PATH variable">>$act_log_file;
   PATH="$TSIEM_HOME/jre/jre/bin:$TSIEM_HOME/jre/ibm-java2-i386-50/jre/bin:$PATH";
fi

LDAPHOST=`grep "SystemName" $PROPS_FILE | awk '{print $3}'`

echo "[`date`] Checking input parameters">>$act_log_file;

if [ -z "$LDAPHOST" ]; 
then
   echo "[`date`] FAILURE: Missing parameter SystemName from $PROPS_FILE">>$act_log_file;
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file;
   exit 1;
fi

if [ -z "$LDAPPORT" ]; 
then
   echo "[`date`] FAILURE: Missing LDAPPORT argument">>$act_log_file;
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file;
   exit 1;
fi

if [ -z "$TEMP_FILE" ]; 
then
   echo "[`date`] FAILURE: Missing OUTPUT_FILE argument ">>$act_log_file;
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file;
   exit 1;
fi

echo "[`date`] Starting user information collect from LDAP">>$act_log_file;
../bin/cmtool -d "$PASSWORD" | \
  java \
    -classpath "../bin/getitdsgroups.jar" \
    com.ibm.tcim.cea.gensub.itdsuis.java.CollectITDSGroups \
    -LDAPHost?$LDAPHOST \
    -LDAPPort?$LDAPPORT \
    -UserName?$USERNAME \
    -Password?"" \
    -StdinPassword \
    -Filename?$TEMP_FILE \
    >>$act_log_file 2>&1;

code=$?
if [ $code = 0 ];
then
   echo "[`date`] SUCCESS: Processing finished" >>$act_log_file;
else
   echo "[`date`] FAILURE: Collect process failed; Exit code "$code>>$act_log_file;
fi
exit $code;
