#!/bin/sh

cea_exit()
{
  debug "$1 `date`"
  echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`) after FAILED collect for the original sublog, `date`." >> $CEA_GENACT_DEBUG_LOG
  rm -f ${TEMPFILE1}_temp 2>>$CEA_GENACT_DEBUG_LOG
  rm -f ${TEMPFILE1} 2>>$CEA_GENACT_DEBUG_LOG
  exit $2
}

export LC_TIME="C"

CEA_AUSEARCH="/sbin/ausearch"
CEA_GENACT_DEBUG_LOG=../log/${PROPSFILE}.log
AUDIT_CONFIG=/etc/audit/auditd.conf
if [ -r ${AUDIT_CONFIG} ]
then
   AUDIT_LOG=`grep "^log_file" ${AUDIT_CONFIG}|sed -e 's/^log_file.*=[         ]*\(.*\)/\1/'`
fi

source ${CEA_GENACT_FUNCLIB} || exit 1

if [[ -x ${CEA_AUSEARCH} && -r ${CEA_AUSEARCH} && -r ${AUDIT_CONFIG} && -r ${AUDIT_LOG} ]]
then
  debug "Have executable access on ${CEA_AUSEARCH}, and read access on audit config and audit log file sudo will not be used."
  CEA_SUDO=""
else
  debug "Do not have executable access on ${CEA_AUSEARCH} or read access on audit config or log file, sudo will be used."
  CEA_SUDO="sudo"
fi

if [ ! -r "${PROPSFILE}" ]
then
  debug "Going to perform initial collect (${PROPSFILE} does not exist)."
  START_DATE=`date --date "now -1 day" +%x`
  [[ $? -ne 0 ]] && cea_exit "Error executing (date --date @${START_TIMESTAMP} +%x)." "102"
  
  CEA_COMMAND="${CEA_SUDO} ${CEA_AUSEARCH} -ts ${START_DATE}"
  debug "Collecting log records of 2 last elapsed days ($CEA_COMMAND). `date`"
else
  START_TIMESTAMP=`cat ${PROPSFILE} | grep "^TIMESTAMP=" | sed -e "s/^TIMESTAMP=\([0-9]*\)$/\1/"`
  if [ -z "${START_TIMESTAMP}" ]
  then
    cea_exit "Error retrieving the timestamp of the last collected record from ${PROPSFILE}." "101"
  fi
  debug "Going to perform non-initial collect of log records that have timestamp equal or after \"${START_TIMESTAMP}\"."
  
  START_DATE=`date --date @${START_TIMESTAMP} +%x`
  [[ $? -ne 0 ]] && cea_exit "Error executing (date --date @${START_TIMESTAMP} +%x)." "102"

  START_TIME=`date --date @${START_TIMESTAMP} +%X`
  [[ $? -ne 0 ]] && cea_exit "Error executing (date --date @${START_TIMESTAMP} +%X)." "103"
  
  CEA_COMMAND="${CEA_SUDO} ${CEA_AUSEARCH} -ts ${START_DATE} ${START_TIME}"
  debug "Collecting the actual log records ($CEA_COMMAND). `date`"
fi

#
# The ausearch command doesn't work properly when there is no terminal. This
# causes the initial collect (self audit) to fail. Apparently ausearch itself
# or some library that it is dependent upon, makes some assumption about fd0
# being the stdin. In order to get around this problem, we need to have stdin
# redirected from /dev/null while running the ausearch command.
#
# 
# -r --raw option is only avaiable in ausearch version 1.3 and above. 
# using 'grep "type="' to get the same result as -r option
#
${CEA_COMMAND} </dev/null | grep "type=" 2>>$CEA_GENACT_DEBUG_LOG >${TEMPFILE1}_temp
if [ $? -ne 0 ]
then
NOMATCHES=`grep "<no matches>" ${CEA_GENACT_DEBUG_LOG}`
[[ -z "${NOMATCHES}" ]] && cea_exit "Error executing ${CEA_AUSEARCH}. `date`" "104"
fi
debug "${CEA_AUSEARCH} execution finished with return code \"$?\". `date`"

if [ ! -r ${TEMPFILE1}_temp ]
then
  CEA_NEW_EVENTS=0
else
  CEA_NEW_EVENTS=`wc -l ${TEMPFILE1}_temp | sed -e 's/^ *\([0-9]\{1,\}\).*/\1/'`
  [[ $? -ne 0 ]] && cea_exit "Error executing wc | sed commands." "105"
fi

if [ ${CEA_NEW_EVENTS} -eq 0 ]
then
  debug "No new records were collected."
  touch ${TEMPFILE1}
  if [ ! -r "${PROPSFILE}" ]
  then
    TIMESTAMP=`date --date "now - 1 hour" +%s`
    [[ $? -ne 0 ]] && cea_exit "Error executing (date --date \"now - 1 hour\" +%s)." "106"
    echo "TIMESTAMP=${TIMESTAMP}">${PROPSFILE}_2
  fi
else
  MAX_TIMESTAMP=`tail -1 ${TEMPFILE1}_temp | sed -e "s/.*msg=audit(\([0-9]*\).*/\1/"`
  [[ $? -ne 0 ]] && cea_exit "Error executing wc | sed commands." "107"  
  debug "Timestamp of the last collected record: \"${MAX_TIMESTAMP}\"."
  
  debug "Leaving log records that have timestamp equal to \"${MAX_TIMESTAMP}\" for the next collect. `date`"
  grep -v "msg=audit(${MAX_TIMESTAMP}" ${TEMPFILE1}_temp>${TEMPFILE1} 2>>$CEA_GENACT_DEBUG_LOG

  debug "Removing temporary files. `date`"
  rm -f ${TEMPFILE1}_temp 2>>$CEA_GENACT_DEBUG_LOG

  echo "TIMESTAMP=${MAX_TIMESTAMP}">${PROPSFILE}_2
fi

echo "CeA/ExtendIT Debug Log finished by $SCRIPTNAME on `hostname` (`uname`) after SUCCESSFUL collect for the original sublog, `date`." >> $CEA_GENACT_DEBUG_LOG
