#!/bin/ksh

TEMPFILE=$1
DBTYPE=$3
DBNAME=$4
DBSCHEMA=$5
PORT=$6
USERNAME=$7
PASSWORD=$8
while [ $# -gt 1 ]; do shift; done
PROPSFILE=$1
ES_LOG="../log/gettamessogroups.err"

if [ -z "$PROPSFILE" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Propsfile type is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1
fi

PROPS_NAME=`basename $PROPSFILE`
PROPS_NAME=${PROPS_NAME%.*}
ES_LOG="../log/${PROPS_NAME}.log"

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing started ..."> $ES_LOG

if [ -z "$TEMPFILE" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Temporary file location is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

DBHOST=`grep "SystemName" $PROPSFILE | awk -F= '{print $2}'`
DBHOST=`echo $DBHOST | sed 's/^ *//'`

if [ -z "$DBHOST" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Missing parameter SystemName from $PROPSFILE" >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi 

echo $DBHOST | grep [\!\?\*\,] > /dev/null
if [ $? -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Host contains illegal characters." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

if [ -z "$PORT" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database port is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

echo $PORT | grep [a-zA-Z\!\?\*\:\.\,] > /dev/null
if [ $? -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database port contains illegal characters." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

if [ -z "$DBNAME" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database name is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi


if [ -z "$USERNAME" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] User name is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

if [ -z "$DBSCHEMA" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database schema is not specified.." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

DBTYPE_OK=0
if [ "$DBTYPE" = "DB2" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ "$DBTYPE" = "Oracle" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ "$DBTYPE" = "MS SQL" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ $DBTYPE_OK -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Wrong database type is specified. Using DB2 by default." >> $ES_LOG
   DBTYPE="DB2"
fi

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Checking Java binary availability." >> $ES_LOG
which java > /dev/null
if [ $? -eq 1 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Java binary is not found." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

echo "SELECT attrValue, roleDescription from (SELECT MAX(roleID) as maxrole, attrValue from (SELECT attrValue, roleID from $DBSCHEMA.IMSIdentityRole inner join $DBSCHEMA.IMSIdentityUniqueAttribute on $DBSCHEMA.IMSIdentityRole.imsID = $DBSCHEMA.IMSIdentityUniqueAttribute.imsID where $DBSCHEMA.IMSIdentityUniqueAttribute.attrName='Enterprise Login') alias GROUP BY attrValue) alias, $DBSCHEMA.IMSRole where maxrole=roleID" > IMSRoles_$PROPSFILE.sql

../bin/cmtool -d "$PASSWORD" | \
  java \
    -Dfile.encoding=UTF-8 \
    -classpath "$CLASSPATH:../bin/jdbc-genact.jar" \
    com.ibm.tcim.cea.gensub.jdbcgenact.java.JdbcGenact \
    -driver="$DBTYPE" \
    -host="$DBHOST" \
    -port="$PORT" \
    -database="$DBNAME" \
    -username="$USERNAME" \
    -passwd="" \
    -props="$PROPSFILE" \
    -outfilename="$PROPSFILE.db_groups" \
    -sqlfilename="IMSRoles_$PROPSFILE.sql" \
    -useheaderline=no  \
    -stdinpassword=yes \
    -separator=";" \
    -eraseoutfile=yes \
    >>$ES_LOG 2>&1

if [ $? -ne 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Error quering database." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

if [ -r "$PROPSFILE.db_groups" ]; then
   echo "[SourceGroup]" > $TEMPFILE
   # sed is used to convert windows-like line terminators to a unix-like
   IFS=';' 
   sed 's/\\/=replacement=/g; s/.$//g' $PROPSFILE.db_groups | \
    while read sqlfile group role trailer; do 
          echo "\"$role role\""
          echo "{" 
          echo "   match logonname \"${group}\"" 
          echo "}" 
    done | sed 's/=replacement=/\\/g' >> $TEMPFILE 
fi

rm -f $PROPSFILE.db_groups IMSRoles_$PROPSFILE.sql $PROPSFILE.tmp

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [SUCCESS]." >> $ES_LOG 



