#!/bin/ksh
TEMP_FILE1=$TEMPFILE1
DBTYPE=$OPTION2
DBNAME=$OPTION3
DBSCHEMA=$OPTION4
PORT=$OPTION5
USERNAME=$OPTION6
PASSWORD=$OPTION7
PROPS_FILE=$PROPSFILE
ES_LOG="../log/gettamessologs.err"

if [ -z "$PROPS_FILE" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Propsfile type is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

PROPS_NAME=`basename $PROPSFILE`
PROPS_NAME=${PROPS_NAME%.*}
ES_LOG="../log/${PROPS_NAME}.log"

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing started ..."> $ES_LOG

if [ -z "$TEMP_FILE1" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Temporary file 1 location is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

DBHOST=`grep "SystemName" $PROPS_FILE | awk -F= '{print $2}'`
DBHOST=`echo $DBHOST | sed 's/^ *//'`

if [ -z "$DBHOST" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Missing parameter SystemName from $PROPS_FILE" >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi 

echo $DBHOST | grep [\!\?\*\,] > /dev/null
if [ $? -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Host contains illegal characters." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

if [ -z "$PORT" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database port is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

echo $PORT | grep [a-zA-Z\!\?\*\:\.\,] > /dev/null
if [ $? -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database port contains illegal characters." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi

if [ -z "$DBNAME" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database name is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

if [ -z "$USERNAME" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] User name is not specified." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

if [ -z "$DBSCHEMA" ]; then
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database schema is not specified.." >> $ES_LOG
  echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
  exit 1;
fi

DBTYPE_OK=0
if [ "$DBTYPE" = "DB2" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ "$DBTYPE" = "Oracle" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ "$DBTYPE" = "MS SQL" ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Database type is $DBTYPE." >> $ES_LOG
   DBTYPE_OK=1
fi
if [ $DBTYPE_OK -eq 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Wrong database type is specified. Using DB2 by default." >> $ES_LOG
   DBTYPE="DB2"
fi

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Checking Java binary availability." >> $ES_LOG
which java > /dev/null
if [ $? -eq 1 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Java binary is not found." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   exit 1;
fi
 
echo "SELECT * FROM $DBSCHEMA.IMSLOGUserActivity" > IMSLOGUserActivity_$PROPS_FILE.sql

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Collecting IMSLOGUserActivity table." >> $ES_LOG
../bin/cmtool -k -d "$PASSWORD" | \
  java \
    -Dfile.encoding=UTF-8 \
    -classpath "$CLASSPATH:../bin/jdbc-genact.jar" \
    com.ibm.tcim.cea.gensub.jdbcgenact.java.JdbcGenact \
    -driver="$DBTYPE" \
    -host="$DBHOST" \
    -port="$PORT" \
    -database="$DBNAME" \
    -username="$USERNAME" \
    -passwd="" \
    -props="$PROPS_FILE" \
    -outfilename="$TEMP_FILE1" \
    -sqlfilename="IMSLOGUserActivity_$PROPS_FILE.sql" \
    -useheaderline=yes \
    -stdinpassword=yes \
    -tdcolumn=logTime \
    -separator=";" \
    -eraseoutfile=yes \
    >>$ES_LOG 2>&1

if [ $? -ne 0 ]; then
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Unable to collect from the database." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [FAILURE]." >> $ES_LOG
   echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Trying to process next sublog." >> $ES_LOG
   exit 0;
fi

if [ ! -f $TEMP_FILE1 ]; then
   touch $TEMP_FILE1
fi

rm -f IMSLOGUserActivity_$PROPS_FILE.sql

echo "[`date "+[%Y-%m-%d %H:%M:%S]"`] Processing finished [SUCCESS]." >> $ES_LOG 
