#!/bin/ksh
TEMP_FILE=$TEMPFILE1
HOST=$OPTION2
PORT=$OPTION3
USER_NAME=$OPTION4
PASSWORD=$OPTION5
TIP_INSTALL_DIR=$OPTION6
PROPS_FILE=$PROPSFILE
act_log_file="../log/`basename $PROPSFILE`.log"

echo "[`date`] Processing start. ">$act_log_file

if [ -z "$PROPS_FILE" ]; then
   echo "[`date`] Failure: Missing property file argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$TEMP_FILE" ]; then
   echo "[`date`] Failure: Missing output file argument">>$act_log_file
   echo "[`date`]  Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$HOST" ]; then
   echo "[`date`] Failure: Missing WAS repository hostname argument">>$act_log_file
   echo "[`date`]  Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$PORT" ]; then
   echo "[`date`] Failure: Missing WAS repository SOAP port argument">>$act_log_file
   echo "[`date`]  Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$USER_NAME" ]; then
   echo "[`date`] Warning: Missing Repository User argument">>$act_log_file
fi

if [ -z "$TIP_INSTALL_DIR" ]; then
   echo "[`date`] Failure: Missing WAS installation directory argument">>$act_log_file
   echo "[`date`]  Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -f "$TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh" ]; then
   echo "[`date`] WebSphere component ($TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh) found.">>$act_log_file
   . $TIP_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh>>$act_log_file
else
   echo "[`date`]  WebSphere component  ('WAS installation directory'/properties/fsdb/_was_profile_default/default.sh) not found. Check 'WAS installation directory' ES property. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -f "$WAS_USER_SCRIPT" ]; then
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> found.">>$act_log_file
   . $WAS_USER_SCRIPT>>$act_log_file
else
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> not found. [FAILURE]">>$act_log_file
   exit 1
fi
                                            
SHOWLOG_PATH="$WAS_HOME/bin/showlog"
TRUSTFILE="$TIP_INSTALL_DIR/profiles/TIPProfile/etc/DummyClientTrustFile.jks"
KEYFILE="$TIP_INSTALL_DIR/profiles/TIPProfile/etc/DummyClientKeyFile.jks"


if [ -z "$CLIENTSSL" ]; then
   # V6.0
   ../bin/cmtool -k -d "$PASSWORD" | \
     "$JAVA_HOME/bin/java" \
       -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
       -Djava.ext.dirs="$JAVA_HOME/jre/lib/ext:$WAS_HOME/plugins:$WAS_EXT_DIRS" \
       -Dserver.root="$WAS_HOME" \
       -Duser.install.root="$USER_INSTALL_ROOT" \
       -classpath "../bin/gettiplogs.jar:$WAS_CLASSPATH:$WAS_HOME/lib/admin.jar:$WAS_HOME/lib/utils.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/lib/j2ee.jar:." \
       com.ibm.tcim.cea.gensub.tsiemtip.java.GetTIPLogs \
       -host $HOST \
       -port $PORT \
       -username $USER_NAME \
       -password "" \
       -trustfile $TRUSTFILE \
       -keyfile $KEYFILE \
       -showlog $SHOWLOG_PATH \
       -chunk $TEMP_FILE \
       -stdinpassword yes \
       -propsfile $PROPS_FILE \
       >>$act_log_file 2>&1
else
   if [ -f "$WAS_HOME/bin/showlog" ]; then
      SHOWLOG_PATH="$WAS_HOME/bin/showlog"
   else
      SHOWLOG_PATH="$WAS_HOME/bin/showlog.sh"
   fi
   # V6.1 and 7.0
   ../bin/cmtool -k -d "$PASSWORD" | \
     "$JAVA_HOME/bin/java" \
       -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
       -Djava.ext.dirs="$JAVA_HOME/jre/lib/ext:$WAS_HOME/plugins:$WAS_EXT_DIRS:." \
       -Dserver.root="$WAS_HOME" \
       "$CLIENTSAS" "$CLIENTSSL" \
       -Duser.install.root="$USER_INSTALL_ROOT" \
       -classpath "../bin/gettiplogs.jar:$WAS_CLASSPATH:$WAS_HOME/lib/admin.jar:$WAS_HOME/lib/utils.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/lib/j2ee.jar:." \
       com.ibm.tcim.cea.gensub.tsiemtip.java.GetTIPLogs \
       -host $HOST \
       -port $PORT \
       -username $USER_NAME \
       -password "" \
       -trustfile $TRUSTFILE \
       -keyfile $KEYFILE \
       -showlog $SHOWLOG_PATH \
       -chunk $TEMP_FILE \
       -stdinpassword yes \
       -propsfile $PROPS_FILE \
       >>$act_log_file 2>&1
fi
if [ $? -eq 0 ]; then
   echo "[`date`] Processing finished. [SUCCESS]">>$act_log_file
else
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
fi
