#!/bin/ksh

TEMPFILE=$TEMPFILE1
LOG_LOCATION=$OPTION2
LOG_TMPL=$OPTION3
JAVA_PATH=$OPTION4

print_message()
{
  echo "$(date "+[%Y-%m-%d %H:%M:%S]") ${1}." >> "${ES_LOG}"
}

failure()
{
  [ -z "${1}" ] || print_message "ERROR: ${1}"
  print_message "Processing finished [FAILURE]"
  exit ${2:-1}
}

PROPS_NAME=`basename $PROPSFILE`
PROPS_NAME=${PROPS_NAME%.*} 
ES_LOG="../log/${PROPS_NAME}.log"; 

>$TEMPFILE
>$ES_LOG

test -z "$LOG_LOCATION" && failure "Missing log location argument"

test -z "$LOG_TMPL" && failure "Missing logfile mask (main log file name) argument"

TIP_HOME=`perl -ne 'if(/^tip\:\d+\:wait:(.*?)\/bin\/startServer/){ print "$1\n" }' < /etc/inittab`
JAVA_SUFFIX='/java/jre/bin/java'

JAVA_PATH=$TIP_HOME$JAVA_SUFFIX
if [ -f "$JAVA_PATH" -a -x "$JAVA_PATH" ]; then
   print_message "Use Java binary file '$JAVA_PATH'"
else
   JAVA_PATH=`which java`
   if [ -f "$JAVA_PATH" -a -x "$JAVA_PATH" ]; then
		print_message "Use Java binary file '$JAVA_PATH'"
   else   
		failure "Java binary file not found"
	fi
fi
 
print_message "Processing started"

"$JAVA_PATH" \
  -classpath "../bin/gettklmlogs.jar" \
  com.ibm.tcim.cea.gensub.itklm.java.GetTKLMLogs \
  "$LOG_LOCATION" \
  "$LOG_TMPL" \
  "$TEMPFILE" \
  "$PROPSFILE" \
  UNIX >>$ES_LOG 2>&1
  
if [ $? = 0 ]; then
   print_message "Processing finished [SUCCESS]"
else
   failure "Collect process failed"
fi
