#!/bin/ksh
# IBM Confidential
# OCO Source Material
# 5724-T46
# (c) Copyright IBM Corp. 2008, 2010
# The source code for this program is not published or otherwise divested of
# its trade secret, irrespective of what has been deposited with the U.S.
# Copyright Office.

TEMP_FILE=$1
HOST=""
PORT=$3
USER_NAME=$4
PASSWORD=$5
WAS_INSTALL_DIR=$6
PROPS_FILE=$7

act_log_file="../log/`basename $PROPS_FILE`.log"

echo "[`date`] Processing start. ">$act_log_file

if [ -z "$PROPS_FILE" ]; then
   echo "[`date`] Failure: Missing property file argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

HOST=`grep "SystemName" $PROPS_FILE | awk '{print $3}'`

if [ -z "$HOST" ]; then
   echo "[`date`] Failure: Missing parameter SystemName from PROPS_FILE">>$act_log_file
   echo "[`date`] Processing finished (Return code 1)">>$act_log_file
   exit 1
fi

if [ -z "$TEMP_FILE" ]; then
   echo "[`date`] Failure: Missing output file argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$PORT" ]; then
   echo "[`date`] Failure: Missing WAS repository SOAP port argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -z "$USER_NAME" ]; then
   echo "[`date`] Warning: Missing Repository User argument">>$act_log_file
fi

if [ -z "$WAS_INSTALL_DIR" ]; then
   echo "[`date`] Failure: Missing WAS installation directory argument">>$act_log_file
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
   exit 1
fi                      

if [ -f "$WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh" ]; then
   echo "[`date`] WebSphere component ($WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh) found.">>$act_log_file
   . $WAS_INSTALL_DIR/properties/fsdb/_was_profile_default/default.sh>>$act_log_file
else
   echo "[`date`] WebSphere component  ('WAS installation directory'/properties/fsdb/_was_profile_default/default.sh) not found. Check 'WAS installation directory' ES property. [FAILURE]">>$act_log_file
   exit 1
fi

if [ -f "$WAS_USER_SCRIPT" ]; then
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> found.">>$act_log_file
   . $WAS_USER_SCRIPT>>$act_log_file
else
   echo "[`date`] WebSphere component <$WAS_USER_SCRIPT> not found. [FAILURE]">>$act_log_file
   exit 1
fi


if [ -z "$CLIENTSSL" ]; then
   # V6.0
   ../bin/cmtool -d "$PASSWORD" | \
     "$JAVA_HOME/bin/java" \
       -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
       -Djava.ext.dirs="$JAVA_HOME/jre/lib/ext:$WAS_EXT_DIRS:$WAS_HOME/plugins: $WAS_HOME/lib/WMQ/java/lib" \
       -Dws.ext.dirs="$WAS_EXT_DIRS;%WAS_USER_DIRS:." \
       -Djava.naming.factory.initial=com.ibm.websphere.naming.WsnInitialContextFactory \
       -Dserver.root="$WAS_HOME" \
       "$CLIENTSAS" \
       -Dcom.ibm.CORBA.loginSource=none \
       -Duser.install.root="$USER_INSTALL_ROOT" \
       -classpath "../bin/getwasgroups.jar:$WAS_CLASSPATH:." \
       com.ibm.tcim.cea.gensub.iwasuis.java.GetWASGroups \
       -host $HOST \
       -port $PORT \
       -username $USER_NAME \
       -password "" \
       -chunk $TEMP_FILE \
       -stdinpassword yes \
       -propsfile $PROPS_FILE \
       >>$act_log_file 2>&1

else
   if [ -z "$CIF_WAS_CLASSPATH" ]; then
      CIF_WAS_CLASSPATH="$WAS_CLASSPATH:$WAS_HOME/plugins/com.ibm.ws.security.crypto_6.1.0.jar:$WAS_HOME/bin/ProfileManagement/plugins/com.ibm.websphere.v61_6.1.0/ws_runtime.jar:$WAS_HOME/bin/ProfileManagement/plugins/com.ibm.websphere.v61_6.1.200/ws_runtime.jar:$WAS_HOME/runtimes/com.ibm.ws.webservices.thinclient_6.1.0.jar:$WAS_HOME/plugins/com.ibm.ws.runtime_6.1.0.jar:$WAS_HOME/plugins/com.ibm.ws.admin.core.jar:$WAS_HOME/plugins/com.ibm.ws.prereq.soap.jar:$WAS_HOME/deploytool/itp/plugins/com.ibm.websphere.v61_6.1.200/ws_runtime.jar:$WAS_HOME/plugins/com.ibm.ws.runtime.jar:$WAS_HOME/plugins/com.ibm.ws.security.crypto.jar:$WAS_HOME/runtimes/com.ibm.ws.webservices.thinclient_7.0.0.jar:$WAS_HOME/plugins/com.ibm.ws.admin.services.jar:$WAS_HOME/lib/bootstrap.jar:$WAS_HOME/plugins/com.ibm.ws.emf.jar:$WAS_HOME/plugins/org.eclipse.emf.ecore.jar:$WAS_HOME/plugins/org.eclipse.emf.common.jar:$WAS_HOME/plugins/com.ibm.ffdc.jar:$WAS_HOME/plugins/com.ibm.ws.prereq.javamail.jar:$WAS_HOME/runtimes/com.ibm.ws.ejb.thinclient_7.0.0.jar:$WAS_HOME/plugins/org.eclipse.osgi_3.2.1.R32x_v20060919.jar"
      echo "[`date`] CIF_WAS_CLASSPATH environment variable is not defined. Default value will be used in CLASSPATH JRE command-line argument." >> $act_log_file
   else
      echo "[`date`] CIF_WAS_CLASSPATH environment variable is defined and will be used in CLASSPATH JRE command-line argument instead of the default value." >> $act_log_file
   fi
   echo "[`date`] CIF_WAS_CLASSPATH=$CIF_WAS_CLASSPATH">>$act_log_file;

   # V6.1 and 7.0
   ../bin/cmtool -d "$PASSWORD" | \
     "$JAVA_HOME/bin/java" \
       -Djava.security.auth.login.config="$WAS_HOME/properties/wsjaas_client.conf" \
       -Dws.ext.dirs="$WAS_EXT_DIRS;$WAS_USER_DIRS:." \
       -Djava.naming.factory.initial=com.ibm.websphere.naming.WsnInitialContextFactory \
       -Dserver.root="$WAS_HOME" \
       "$CLIENTSAS" \
       "$CLIENTSSL" \
       -Duser.install.root="$USER_INSTALL_ROOT" \
       -classpath "../bin/getwasgroups.jar:$CIF_WAS_CLASSPATH:." \
       com.ibm.tcim.cea.gensub.iwasuis.java.GetWASGroups \
       -host $HOST \
       -port $PORT \
       -username $USER_NAME \
       -password "" \
       -chunk $TEMP_FILE \
       -stdinpassword yes \
       -propsfile $PROPS_FILE \
       >>$act_log_file 2>&1
fi

if [ $? -eq 0 ]; then
   echo "[`date`] Processing finished. [SUCCESS]">>$act_log_file
else
   echo "[`date`] Processing finished. [FAILURE]">>$act_log_file
fi
