#!/usr/bin/perl -w

my ($line,$size,%data);

sub Usage {
  print "This script will convert a canned BIM/Alert log file for usage with ISM.\n";
  print "usage: $0 \"canned report\"\n";
}

sub trim {
  my @out = @_;
  for (@out) {
    s/^\s+//;
    s/\s+$//;
  }
  return wantarray ? @out : $out[0];
}

sub expand {
  my $out = $_[0];
  my ($i);
  my $lng = 0;
  $lng = length($out) if $out;
  for ($i = $lng; $i < 20; $i++) {
    $out .= " ";
  }
  return $out;
}

die &Usage unless ($ARGV[0] && (-e $ARGV[0]));
my $logfile = $ARGV[0];
my $dataset = 0;
open(LOGF, "< $logfile") or die "Couldn't open $logfile for reading: $!\n";
while (defined ($line = <LOGF>)) {
  chomp $line;
  if ($line =~ /^.DATE\: /) {next;};
  if ($line =~ /^.TIME\: /) {next;};
  if ($line =~ /^.                                                    \* \* SECURITY AUDIT REPORT/) {next;};
  if ($line =~ /^.             DATE      TIME/) {next;};
  if ($line =~ /^.              OF        OF    SYS       ACTION/) {next;};
  if ($line =~ /^.\.\.ADMIN\.\.   CHANGE    CHANGE /) {next;};
  if ($line =~ /^.{71}NO CHANGE/) {next;};
  if ($line =~ /^.([\w\s]{9}) (\d+\/\d+\/\d+ \d+:\d+:\d+) ([\w\s]{4}) ([\w\s]{4}) ([\w\s]{8}) (DATASET\=.*)$/) { 
    $data{'opernum'} = $1;
    $data{'datetime'} = $2;
    $data{'sysid'} = $3;
    $data{'trantype'} = $4;
    $data{'action'} = $5;
    $data{'dataset'} = $6;
    $dataset = 1;
    next;
  }
  if (($dataset == 1) and (($line =~ /^ {49} (.*)$/) or
      ($line =~ /^.[\w\s]{9} \d+\/\d+\/\d+ \d+:\d+:\d+ \(\*\*\* CONTINUED \*\*\*\)(.*)$/))) { 
    $data{'dataset'} .= ",$1";
    $dataset++;
    next;
  }
  if (($dataset == 2) and (($line =~ /^ {49} (.*)$/) or
      ($line =~ /^.[\w\s]{9} \d+\/\d+\/\d+ \d+:\d+:\d+ \(\*\*\* CONTINUED \*\*\*\)(.*)$/))) { 
    $data{'dataset'} .= ",$1";
    $dataset = 0;
    print "$data{'opernum'} $data{'datetime'} $data{'sysid'} $data{'trantype'} $data{'action'} $data{'dataset'}\n";
    next;
  }
  if ($line =~ /^.([\w\s]{9}) (\d+\/\d+\/\d+ \d+:\d+:\d+) ([\w\s]{4}) ([\w\s]{4}) ([\w\s]{8}) (.{1,20})/) { 
    $data{'opernum'} = $1;
    $data{'datetime'} = $2;
    $data{'sysid'} = $3;
    $data{'trantype'} = $4;
    $data{'action'} = $5;
  }
  if ($line =~ /^.([\w\s]{9}) (\d+\/\d+\/\d+ \d+:\d+:\d+) \(\*\*\* CONTINUED \*\*\*\)(.{1,20})/) { 
    $data{'opernum'} = $1;
    $data{'datetime'} = $2;
  }
  print "$data{'opernum'} $data{'datetime'} $data{'sysid'} $data{'trantype'} $data{'action'} ";
  if ($line =~ /^.{50}([\w\s]{20}) (.{20}) (.{20}) (.{1,20})$/) {
    #Field changed to
    $data{'resource'} = $1 unless (&trim($1) eq '');
    $data{'field'} = $2 unless (&trim($2) eq '');
    $data{'from'} = $3;
    $data{'to'} = $4;
  }
  if ($line =~ /^.{50}([\w\s]{20}) (.{20}) (.{1,20})$/) {
    #Field changed from
    $data{'resource'} = $1 unless (&trim($1) eq '');
    $data{'field'} = $2 unless (&trim($2) eq '');
    $data{'from'} = $3;
    $data{'to'} = '';
  }
  if ($line =~ /^.{50}([\w\s]{20}) (.{1,20})$/) {
    #Field changed
    $data{'resource'} = $1 unless (&trim($1) eq '');
    $data{'field'} = $2 unless (&trim($2) eq '');
    $data{'from'} = '';
    $data{'to'} = '';
  }
  if ($line =~ /^.{50}([\w\s]{1,20})$/) {
    #Field changed
    $data{'resource'} = $1 unless (&trim($1) eq '');
    $data{'field'} = '';
    $data{'from'} = '';
    $data{'to'} = '';
  }
  $data{'resource'} = &expand($data{'resource'});
  $data{'field'} = &expand($data{'field'});
  $data{'from'} = &expand($data{'from'});
  $data{'to'} = &expand($data{'to'});
  print "$data{'resource'} $data{'field'} $data{'from'} $data{'to'}\n";
}
close(LOGF);
 