LC_ALL=C
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#!/bin/sh
###########################################################BeginFile############
# Copyright:  
#   Copyright (c) IBM Corp. 1998,2007. All Rights Reserved.
#
# File-stamp: <070613 Luc 09:49 zOS_collect_groups.sh>
#
# Name:
#   zOS_collect_groups.sh
#
# Purpose:
#   Generic actuator script for the z/OS platform.
#   It collects grouping information.
#
# Notes:
#   The used command interpreter (shell) needs to support
#   the [[ expression ]] construct.
#   The z/OS shell, based on the KornShell, supports this construct.
#   Part of this script is based on the HPUX genact.sh script.
#
# History:
#   050831 0.0.1 luc EE0501008: Created.
#   051109 0.0.3 luc EE0501008: Extended ES and UIS configuration
#   051111 0.0.4 luc QE0412060: Added TCB ObjectGroups to UIS
#   051124 0.0.5 luc QE0412060: Fixed ObjectGroups "match" syntax
#   051125 0.0.5 luc EE050B016: Log script start/end to console
#   060123 0.0.6 luc QE0412060: Changed and added some TCB ObjectGroups
#   060203 0.0.8 Luc EE0501008: Improved error handling
#   060220 0.0.14 luc EE050B004: Separation of UIS configuration and collection
#   060301 0.0.21 luc UE050B008: Added newlist names
#   060306 0.0.22 luc EE0501008: Do not require UNLOAD with POLL and WAIT
#   060308 0.0.26 luc EE050B016: Send messages to the SDSF system log
#   060310 0.0.28 luc EE0501008: Alloc type=acf2 primary active
#   060322 0.0.35 luc UZ0602037: Generate <ESM> groups only on <ESM> systems
#   060324 0.0.37 luc UZ0603009: Optionally use APF-authorized CARLa
#   060327 0.0.39 luc QE0412060: Added port to system log messages
#   060331 0.0.41 luc WE0603019: Keep SMF when another ES is LIVE
#   060403 0.0.43 luc WE0603019: Put agent information in MASTER data set
#   060403 0.0.44 luc WE0603019: Replace wc -c with ls -l
#   060410 0.0.47 luc WE0603019: Refresh MASTER data set more often;
#                                skip duplicate IOCONFIGs
#   060410 0.0.49 luc WE0603019: Move PORT in front of logger messages
#   060412 0.0.52 luc QE0604034: Compress UIS
#   060413 0.0.53 luc WE0604016: Report "compressed" or "uncompressed"
#   060413 0.0.54 luc WE0604016: Report bytes instead of megabytes
#   060424 0.0.57 luc WE0603019: Handle empty UIS/ES property directories
#   060505 0.0.60 luc EZ0604016: Customized CARLa inclusion
#   061110 0.0.73 luc QE0606008: Set directory permission bits
#   061117 0.0.74 luc QE06S7025: Changed C2E012I, C2E013I, and C2E111I wording
#   070419 0.0.75 Luc EE0704005: Light blue wash
#   070521 0.0.76 Luc EE0704005: Copy CKFREEZE to IOCONFIG properties
#   070613 0.0.77 Luc EE0704005: Use sc2rsamp with zSecure < 1.8.1
###########################################################EndFile##############

# Terminate function which removes temporary files, optionally prints a message to $STDERR, and exits.
# This function should only be called after defining $STDERR.
terminate ()
{
    rm -rf $RUN_SUBDIR
    touch $STDERR # remains empty if there is no error
    [[ -n "$3" ]] && echo "`date`: `basename $0`: $PORT: $3" >> $STDERR
    [[ -n "$3" ]] && logger -a "C2E$2I: $PORT" "$3"
    logger -a "C2E002I: $PORT" "End of `basename $0` with rc=$1; id=$ID"
    [[ $1 -ne 0 ]] && logger -a "C2E003I: $PORT" "Agent root is `dirname $PWD`"
    exit $1
}

# The next function does not log to the $STDERR file
early_terminate ()
{
    [[ -n "$3" ]] && echo "`date`: `basename $0`: $PORT: $3"
    [[ -n "$3" ]] && logger -a "C2E$2I: $PORT" "$3"
    logger -a "C2E002I: $PORT" "End of `basename $0` with rc=$1; id=$ID"
    [[ $1 -ne 0 ]] && logger -a "C2E003I: $PORT" "Agent root is `dirname $PWD`"
    exit $1
}

store_properties ()
{
    if [[ "$LAST_SUCCESS_DATE" == "" ]]
    then
        # Read previous last successful collect date (may be empty)
        touch $PARAMETER_FILE  # make sure the next command does not fail
        LAST_SUCCESS_DATE=`awk -F= '/Last successful collect/ {print $2}' $PARAMETER_FILE`
    fi
    # Store properties and last (successful) collect dates
    echo "Configuration file=$GROUP_CFG_FILE" > $PARAMETER_FILE
    echo "Collect strategy=$STRATEGY" >> $PARAMETER_FILE
    echo "Store raw files=$RAW" >> $PARAMETER_FILE
    echo "Last collect=$NOW" >> $PARAMETER_FILE
    echo "Last successful collect=$LAST_SUCCESS_DATE" >> $PARAMETER_FILE
    echo "Error retention=$RETPD" >> $PARAMETER_FILE
    echo "Complex name=`echo $COMPLEX | awk -F= '{print $2}'`" >> $PARAMETER_FILE
    echo "CKFREEZE Data Set Name=$CKFREEZE" >> $PARAMETER_FILE
    echo "IOCONFIG Data Set Name=$CKFREEZE" >> $PARAMETER_FILE     # enable reversion to software which expects IOCONFIG
    echo "UNLOAD Data Set Name=$UNLOAD" >> $PARAMETER_FILE
    echo "System Policy Type=$POLICY_TYPE" >> $PARAMETER_FILE
}

refresh_master ()
{
    # Refresh master data set (including information just written to $PARAMETER_FILE)
    echo "[Agent]" > $MASTER_CONTENTS
    # Determine agent ID
    # This assumes ../bin/db2asc ../run/startup has a single line which contains " 1.2.3.15(int)" and which looks like ... <id>(..., e.g.
    # ((objid) 1.2.3.15(int) 1) = (((uint) 80(objval) ((objid) 12.1.136(objid) 1.2.3.14(objval) ((objval) ((uint) 1(uint) 4)))))
    echo "Id=`../bin/db2asc ../run/startup | grep ' 1.2.3.15(int)' | awk -F\  '{print $8}' | awk -F\( '{print $1}'`" >> $MASTER_CONTENTS
    # Determine server name (or IP) and port
    # This assumes ../bin/db2asc ../run/startup has a single line which contains ""A" and which looks like ...:<name or IP>:<port number>"..., e.g.
    # ((objid) 1.2.3.4(int) 3) = (((uint) 1920(objval) ((int) 1(string) "12.1.1"(string) "0"(string) "A:CRMNLW016:5050"(objval) ())))
    SERVER=`../bin/db2asc ../run/startup | awk -F: '/\"A/ {print $2 "\"" $3}' | awk -F\" '{print $1 ":" $2}'`
    echo "Server=$SERVER" >> $MASTER_CONTENTS
    CFG=""
    [[ -f ../run/configuration_file_name ]] && CFG=`cat ../run/configuration_file_name`
    echo "ConfigFile=$CFG" >> $MASTER_CONTENTS
    for ES_STATE in `ls $ES_STATES | grep -v cutoff`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[EventSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $ES_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS             # ES_STATE = <x>.<y>.<z>.props
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "SMF=`awk -F= '/SMF Data Set Name/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Collect_Past_Data=`awk -F= '/Collect past data \(discouraged\)/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
    done
    for UIS_STATE in `ls $UIS_STATES`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[UserInformationSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $UIS_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS        # UIS_STATE = <x>.<y>.<z>.props
      echo "Complex=`awk -F= '/Complex name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "CKFREEZE=`awk -F= '/CKFREEZE Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "UNLOAD=`awk -F= '/UNLOAD Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
    done
    cp $MASTER_CONTENTS "//'$MASTER'"
}

# Determine agent port number
# This assumes ../bin/db2asc ../run/startup has a single line which contains "listen" and which looks like ...:...:<port number>"..., e.g.
# ((string) "listen") = (((string) "0"(string) "P:DEST_ANY:7300"(objval) ()))
PORT=`../bin/db2asc ../run/startup | awk -F: '/\"listen\"/ {print $3}' | awk -F\" '{print $1}'`

# Check if this script is executed within the correct working directory.
[[ `basename $PWD` = "run" && -d ../bin && -d ../log ]] || early_terminate 8 004 "$0 must be executed from the run directory"

# Check number of arguments
[[ $# -ge 10 ]] || early_terminate 8 005 "$0 expects at least 10 arguments"

# Determine output and log directory prefix
STATE_PREFIX=${10}
ID=`echo $STATE_PREFIX | awk -F. '{print $1 "." $2 "." $3}'` # <x>.<y>.<z>.props

# Log script start to console
logger -a "C2E001I: $PORT" "Start of `basename $0`; id=$ID"
logger -a "C2E003I: $PORT" "Agent root is `dirname $PWD`"

# Determine user information source and event source state directories
UIS_STATES="../run/UIS_properties"
ES_STATES="../run/ES_properties"

# Determine user information source parameter file
PARAMETER_FILE="$UIS_STATES/$STATE_PREFIX"

# Determine file name components
RUN_SUBDIR="$PWD/../run/$STATE_PREFIX.tmp"
LOG_SUBDIR="$PWD/../log/$STATE_PREFIX"
NOW=`date +%Y-%m-%d.%H:%M:%S`

# Create directories if they are not yet there.
[[ -d $LOG_SUBDIR ]] || mkdir -p $LOG_SUBDIR
[[ -d $RUN_SUBDIR ]] || mkdir -p $RUN_SUBDIR
[[ -d $UIS_STATES ]] || mkdir $UIS_STATES
[[ -d $ES_STATES ]] || mkdir $ES_STATES

# Set directory permission bits
getfacl ../log | setfacl -S - $LOG_SUBDIR
getfacl ../run | setfacl -S - $RUN_SUBDIR
getfacl ../run | setfacl -S - $UIS_STATES
getfacl ../run | setfacl -S - $ES_STATES

# Determine file names
STDOUT="$LOG_SUBDIR/$NOW.sh.stdout"
STDERR="$LOG_SUBDIR/$NOW.sh.stderr"
if [ -f CKR ]
then
  CKRCARLA="CKRCARLA"
  CKRCARLX="CKRCARLx"
  CKFCOLL="CKFCOLL"
  CKFCOLLX="CKFCOLL"
  SCKRCARL="sckrcarl"
  SAMPLES="ckrcarla"
  CKRDSN="ckrdsn"
  CKRPREFIX="CKR"
  CKFPREFIX="CKF"
  TCKFREEZE="CKFREEZE"
else
  CKRCARLA="C2RCARLA"
  CKRCARLX="C2RCARLx"
  CKFCOLL="C2FOSCOL"
  CKFCOLLX="C2FOSCOL"
  SCKRCARL="sc2rsamp"
  SAMPLES="cnrsamp"
  CKRDSN="c2rdsn"
  CKRPREFIX="CNR"
  CKFPREFIX="CNF"
  TCKFREEZE="IOCONFIG"
fi
SYSPRINT="$LOG_SUBDIR/$NOW.$CKRCARLA.SYSPRINT"
COLPRINT="$LOG_SUBDIR/$NOW.$CKFCOLL.SYSPRINT"
CPU="$RUN_SUBDIR/CPU"                                       # tmp
GROUPS_CONFIG="$RUN_SUBDIR/groups_config"                   # tmp
GROUPS_CONFIG_RACF="$RUN_SUBDIR/groups_config_racf"         # tmp
GROUPS_CONFIG_ACF2="$RUN_SUBDIR/groups_config_acf2"         # tmp
MASTER_CONTENTS="$RUN_SUBDIR/master_contents"               # tmp
SOURCE_GROUPS_RACF="$RUN_SUBDIR/source_groups_racf"         # tmp
SOURCE_GROUPS_ACF2="$RUN_SUBDIR/source_groups_acf2"         # tmp

# Make sure the next files exist, so information can be appended to them.
touch $STDOUT
touch $STDERR

# Determine MASTER and ENQ data set names, as well as some of their qualifiers
MASTER=`cat ../run/master`
[[ "$MASTER" != "" ]] || terminate 8 006 "The master file in the run directory does not exist or is empty"
C2ELVPFX=`echo $MASTER | awk -F. '{prefix = ""}{for(i=1; i<=NF && $i != "MASTER"; i++) {prefix = prefix $i "."}}{print prefix}'`
MASTERQUAL=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)} {if(i <= NF) {print "MASTER"}}'`
[[ "$MASTERQUAL" != "" ]] || terminate 8 007 "The master data set specified in the master file in the run directory should have a MASTER qualifier"
C2ELVLLQ=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)}{suffix = ""}{for(j=i+1; j<=NF; j++){suffix = suffix "." $j}}{print suffix}'`
ENQ="${C2ELVPFX}ENQ$C2ELVLLQ"
DEFAULT_CKFREEZE="${C2ELVPFX}${TCKFREEZE}$C2ELVLLQ"

# Read and check temporary file parameter (which is never passed to zOS_collect_config.sh)
TEMP_FILE1=$1
[[ ! -z "$TEMP_FILE1" ]] || terminate 8 008 "Name of Temporary file 1 is not given."

# Read properties
GROUP_CFG_FILE=$2
STRATEGY=$3
RAW=$4
RETPD=$5
COMPLEX=$6
CKFREEZE=$7
UNLOAD=$8
POLICY_TYPE=$9

# Convert some properties to uppercase
STRATEGY=`echo $STRATEGY | tr '[:lower:]' '[:upper:]'`
RAW=`echo $RAW | tr '[:lower:]' '[:upper:]'`
COMPLEX=`echo $COMPLEX | tr '[:lower:]' '[:upper:]'`
CKFREEZE=`echo $CKFREEZE | tr '[:lower:]' '[:upper:]'`
UNLOAD=`echo $UNLOAD | tr '[:lower:]' '[:upper:]'`
POLICY_TYPE=`echo $POLICY_TYPE | tr '[:lower:]' '[:upper:]'`
# Convert POLICY_TYPE to Production or Test
[[ "$POLICY_TYPE" == "PRODUCTION" ]] && POLICY_TYPE="Production"
[[ "$POLICY_TYPE" == "TEST" ]] && POLICY_TYPE="Test"

# Check properties
if [[ "$STRATEGY" != "LIVE" && "$STRATEGY" != "POLL" && "$STRATEGY" != "WAIT" ]]
then
    terminate 8 009 "The \"Collect strategy\" parameter is $STRATEGY but should have been LIVE, POLL, or WAIT."
fi

[[ "$RAW" == "YES" || "$RAW" == "NO" ]] || terminate 8 010 "The \"Store raw files\" parameter is $RAW but should have been YES or NO."

[[ ! -z "$RETPD" ]] || terminate 8 011 "An error retention period should have been specified."

if [[ "$STRATEGY" == "LIVE" ]]
then
    [[ -z "$CKFREEZE" ]] || terminate 8 012 "With \"Collect strategy\" $STRATEGY, a $TCKFREEZE specification is not allowed."
    CKFREEZE=$DEFAULT_CKFREEZE
    [[ -z "$UNLOAD" ]] || terminate 8 013 "With \"Collect strategy\" $STRATEGY, an UNLOAD specification is not allowed."
else # POLL or WAIT
    [[ ! -z "$CKFREEZE" ]] || terminate 8 014 "With \"Collect strategy\" $STRATEGY, a $TCKFREEZE must be specified."
fi

if [[ "$POLICY_TYPE" != "Production" && "$POLICY_TYPE" != "Test" ]]
then
    terminate 8 015 "The \"System Policy Type\" parameter is $POLICY_TYPE but should have been Production or Test."
fi

# Convert the COMPLEX property to a CARLa phrase
[[ "$COMPLEX" != "" ]] && COMPLEX="complex=$COMPLEX"

# Make sure there is not a more recently configured UIS with the same CKFREEZE as the UIS currently being collected
CHECK_CKF=0
for UIS_STATE in `ls -tr $UIS_STATES`
do
  if [[ $CHECK_CKF -eq 1 && "$CKFREEZE" == `awk -F= '/CKFREEZE Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE` ]]
  then
    terminate 8 016 "Did not collect $STATE_PREFIX, sharing $CKFREEZE with $UIS_STATE"
  fi
  [[ "$UIS_STATE" == "$STATE_PREFIX" ]] && CHECK_CKF=1
done

# Store properties and refresh master data set (so current data are known even if Collect or CARLa fails)
store_properties
refresh_master

# Locate license and sample library
[[ -f CKR || -s ../run/license ]] || terminate 8 017 "The license file in the run directory does not exist or is empty"
[[ -s ../run/$SCKRCARL ]] || terminate 8 018 "The $SCKRCARL file in the run directory does not exist or is empty"

# Run zCollect if the strategy is LIVE
if [[ "$STRATEGY" == "LIVE" ]]
then
    if [ -f CKR ]
    then
      MAYBELICENSE=
    else
      MAYBELICENSE="license='`cat ../run/license`'"
    fi
    ../run/$CKFCOLLX "$MAYBELICENSE $TCKFREEZE='$CKFREEZE' serialization(wait,maxwait(30),enq($CKRDSN,sysdsn))" < /dev/null > $COLPRINT 2> $STDERR
    RETURN_CODE=$?
    [[ $RETURN_CODE -lt 12 ]] || terminate 8 019 "Collect groups error occurred in $CKFCOLL - see `basename $COLPRINT`"
# The next line assumes the CKF034I message looks like CKF034I 00 CKFCOLL used 19.5 CPU seconds, 49 elapsed seconds, and collected 27.935 MB (0.568 MB/s)
    grep -e "^${CKFPREFIX}034I 00" $STDERR | head -n 1 > $CPU
    echo "`date`: Collect took `awk '{print $5}' $CPU` CPU seconds, collected `awk '{printf "%0.0f MB", $13}' $CPU`, and took `awk '{print $8}' $CPU` wall clock seconds" >> $STDOUT
fi

rm -f $GROUPS_CONFIG
[[ -f CKR ]] || echo "alloc type=input dd=license dsn='`cat ../run/license`'" >> $GROUPS_CONFIG
echo "alloc type=input dd=$SAMPLES dsn='`cat ../run/$SCKRCARL`'" >> $GROUPS_CONFIG

# Embed custom sample library
[[ `cat ../run/c2esamp` != "" ]] && echo "alloc type=input dd=c2esamp dsn='`cat ../run/c2esamp`'" >> $GROUPS_CONFIG
echo "alloc type=input dd=c2ei0uis path='$PWD/../run/c2ei0uis'" >> $GROUPS_CONFIG
if [[ "$STRATEGY" == "POLL" ]]
then
    echo "option serialization(fail,enq($CKRDSN,sysdsn))" >> $GROUPS_CONFIG
else
    echo "option serialization(wait,maxwait(30),enq($CKRDSN,sysdsn),unit,volser)" >> $GROUPS_CONFIG
fi

echo "alloc type=$TCKFREEZE dsn=$CKFREEZE $COMPLEX" >> $GROUPS_CONFIG


if [[ "$UNLOAD" != "" ]]
then
    echo "alloc type=unload dsn=$UNLOAD $COMPLEX" >> $GROUPS_CONFIG
else
    echo "alloc type=racf primary active $COMPLEX" > $GROUPS_CONFIG_RACF
    echo "alloc type=acf2 primary active $COMPLEX" > $GROUPS_CONFIG_ACF2
    echo "include esm=racf path='$GROUPS_CONFIG_RACF'" >> $GROUPS_CONFIG
    echo "include esm=acf2 path='$GROUPS_CONFIG_ACF2'" >> $GROUPS_CONFIG      
fi

UN="un"
# 060410 0.0.51 louis EE0604001: if temp file name ends in .gz then
# contents must be gzipped
case $TEMP_FILE1 in
 *.gz) COMPRESS="compress=gzip"
       UN="";;
esac

echo "alloc type=output dd=groups path='$TEMP_FILE1'" >> $GROUPS_CONFIG
echo "fileoption dd=groups encoding=utf-8 $COMPRESS" >> $GROUPS_CONFIG
echo "i m=c2eluis" >> $GROUPS_CONFIG

cat >> $GROUPS_CONFIG <<EOF
newlist type=system name=ObjGpHd2 outlim=1
  sortlist / '[ObjectGroup]'

newlist type=system name=ReadSens outlim=1
  sortlist '"Read Sensitive Data - $POLICY_TYPE"' /,
    '{' /,
    '  in obj_group "z/OS READ Sensitive Data - System ' | system(0) | '"' /,
    '}'

newlist type=system name=WritSens outlim=1
  sortlist '"Write Sensitive Data - $POLICY_TYPE"' /,
    '{' /,
    '  in obj_group "z/OS UPDATE Sensitive Data - System ' | system(0) | '"' /,
    '}' /,
    '{' /,
    '  in obj_group "z/OS READ Sensitive Data - System ' | system(0) | '"' /,
    '}'

newlist type=system name=CrDlSens outlim=1
  sortlist '"CreateDelete Sensitive Data - $POLICY_TYPE"' /,
    '{' /,
    '  in obj_group "z/OS ALTER Sensitive Data - System ' | system(0) | '"' /,
    '}'
EOF

# Then there are some RACF and ACF2 SourceGroups
cat >> $SOURCE_GROUPS_RACF <<EOF
newlist type=system name=SrcGrpH2 outlim=1                    
  sortlist / '[SourceGroup]'

newlist type=system name=GrpHdrs outlim=1
  sortlist '"Systems Admin - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "z/OS RACF SYS1 connected - $POLICY_TYPE"' /,
    '}' /,
    '{' /,
    '  in src_group "z/OS RACF Operations - $POLICY_TYPE"' /,
    '}' /,
    '"Security Admin - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "z/OS RACF Special - $POLICY_TYPE"' /,
    '}' /,
    '"Auditor"' /,
    '{' /,
    '  in src_group "z/OS RACF Auditor"' /,
    '}' /,
    '"z/OS RACF Auditor"' /,
    '{' /,
    '  in src_group "z/OS RACF Auditor - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS RACF SYS1 connected - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.RACF SYS1 connected - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS RACF Operations - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.RACF Operations - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS RACF Special - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.RACF Special - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS RACF Auditor - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.RACF Auditor - $POLICY_TYPE"' /,
    '}'
                                                              
newlist type=system name=RACFSYS1 outlim=1
  sortlist '"' | complex(0) | '.RACF SYS1 connected - $POLICY_TYPE"' /,
    '{' /,                                                       
    '  in src_group "' | complex(0) | '.SYS1"' /,                
    '}' /,
    '"' | complex(0) | '.SYS1"' /* defines the group in case no users are connected to SYS1 */

newlist type=system name=OperHdr outlim=1                     
  sortlist '"' | complex(0) | '.RACF Operations - $POLICY_TYPE"'             
                                                              
newlist type=racf name=Oper                                   
  select class=user segment=base operations                   
  sortlist '{' /,                                             
    '  match logonname "' | complex(0) | '.' | key(0) | '"' /,
    '}'                                                       
                                                              
newlist type=system name=SpecHdr outlim=1                     
  sortlist '"' | complex(0) | '.RACF Special - $POLICY_TYPE"'                
                                                              
newlist type=racf name=Special                                
  select class=user segment=base special                      
  sortlist '{' /,                                             
    '  match logonname "' | complex(0) | '.' | key(0) | '"' /,
    '}'                                                       
                                                              
newlist type=system name=AudHdr outlim=1                      
  sortlist '"' | complex(0) | '.RACF Auditor - $POLICY_TYPE"'                
                                                              
newlist type=racf name=Auditor                                
  select class=user segment=base auditor                      
  sortlist '{' /,                                             
    '  match logonname "' | complex(0) | '.' | key(0) | '"' /,
    '}'                                                       
EOF

cat >> $SOURCE_GROUPS_ACF2 <<EOF
newlist type=system name=SrcGrpH2 outlim=1                    
  sortlist / '[SourceGroup]'

newlist type=system name=GrpHdrs outlim=1
  sortlist '"Systems Admin - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "z/OS ACF2 Non-cncl - $POLICY_TYPE"' /,
    '}' /,
    '"Security Admin - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "z/OS ACF2 Unscoped Security - $POLICY_TYPE"' /,
    '}' /,
    '"Auditor"' /,
    '{' /,
    '  in src_group "z/OS ACF2 Auditor"' /,
    '}' /,
    '"z/OS ACF2 Auditor"' /,
    '{' /,
    '  in src_group "z/OS ACF2 Unscoped Auditor - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS ACF2 Non-cncl - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.ACF2 Non-cncl - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS ACF2 Unscoped Security - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.ACF2 Unscoped Security - $POLICY_TYPE"' /,
    '}' /,
    '"z/OS ACF2 Unscoped Auditor - $POLICY_TYPE"' /,
    '{' /,
    '  in src_group "' | complex(0) | '.ACF2 Unscoped Auditor - $POLICY_TYPE"' /,
    '}'
                                                              
newlist type=system name=ACF2NClH outlim=1
  sortlist '"' | complex(0) | '.ACF2 Non-cncl - $POLICY_TYPE"'             
                                                              
newlist type=acf2_lid name=ACF2NCcl                          
  select non-cncl                                             
  sortlist '{' /,                                             
    '  match logonname "' | complex(0) | '.' | lid(8) | '"' /,
    '}'                                                       
                                                              
newlist type=system name=ACF2UScH outlim=1                       
  sortlist '"' | complex(0) | '.ACF2 Unscoped Security - $POLICY_TYPE"'                   
                                                                 
newlist type=acf2_lid name=ACF2USec                              
  select security missing(scplist)                               
  sortlist '{' /,                                                
    '  match logonname "' | complex(0) | '.' | lid(8) | '"' /,   
    '}'                                                          
                                                                 
newlist type=system name=ACF2UAuH outlim=1                    
  sortlist '"' | complex(0) | '.ACF2 Unscoped Auditor - $POLICY_TYPE"'                
                                                              
newlist type=acf2_lid name=ACF2UAud                            
  select audit missing(scplist)                               
  sortlist '{' /,                                             
    '  match logonname "' | complex(0) | '.' | lid(8) | '"' /,
    '}'                                                       
EOF

echo "include esm=racf path='$SOURCE_GROUPS_RACF'" >> $GROUPS_CONFIG
echo "include esm=acf2 path='$SOURCE_GROUPS_ACF2'" >> $GROUPS_CONFIG

# Run CARLa
../run/$CKRCARLX "i path='$GROUPS_CONFIG'" < /dev/null > $SYSPRINT 2>> $STDERR
RETURN_CODE=$?

[[ $RETURN_CODE -ge 12 ]] && terminate 8 020 "Collect groups error occurred in $CKRCARLA - see messages above (and `basename $SYSPRINT`)"

LAST_SUCCESS_DATE=$NOW
# Store properties and refresh master data set - including an updated LAST_SUCCESS_DATE
store_properties
refresh_master

# The next lines assume the CNR0039 message looks like "CNR0039 00 <program_name> used 3.13 CPU seconds, 5,248KB, and took 9.5 wall clock seconds".
# A conversion of "5,248KB," to an integer number of megabytes is possible but the $TEMP_FILE1 size may be more interesting and is reported now.
# It is also assumed that ls -l $TEMP_FILE1 output looks like "... ... ... ... <size> ......".
grep -e "^${CKRPREFIX}0039 00" $STDERR | head -n 1 > $CPU
echo "`date`: Collect groups script took `awk '{print $5}' $CPU` CPU seconds, `ls -l $TEMP_FILE1 | awk '{print $5}'` bytes ${UN}compressed, and `awk '{print $11}' $CPU` wall clock seconds" >> $STDOUT

# remove log files older than $RETPD days
find $LOG_SUBDIR -type f -mtime +$RETPD | xargs rm

# Clean up and exit with success.
terminate 0
