LC_ALL=C
# Licensed Materials - Property of IBM
# 5724-S67
# (c) Copyright IBM Corp. 1998, 2007   All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#!/bin/sh
###########################################################BeginFile############
# Copyright:  
#   Copyright (c) IBM Corp. 1998,2007. All Rights Reserved.
#
# File-stamp: <070521 Luc 09:36 zOS_config_groups.sh>
#
# Name:
#   zOS_config_groups.sh
#
# Purpose:
#   To configure a z/OS User Information Source.
#
# Notes:
#   The script creates a parameter file and updates a master data set.
#
# History:
#   051109 0.0.3 luc EE0501008: Extended ES and UIS configuration
#   060203 0.0.8 Luc EE0501008: Improved error handling; support ACF2 databases
#   060220 0.0.14 luc EE050B004: Separation of UIS configuration and collection
#   060308 0.0.26 luc EE050B016: Send messages to the SDSF system log
#   060327 0.0.39 luc QE0412060: Added port to system log messages
#   060331 0.0.41 luc WE0603019: Keep SMF when another ES is LIVE
#   060410 0.0.47 luc WE0603019: Refresh MASTER data set more often
#   060410 0.0.49 luc WE0603019: Move PORT in front of logger messages
#   060424 0.0.57 luc WE0603019: Handle empty UIS/ES property directories
#   061110 0.0.73 luc QE0606008: Set directory permission bits
#   070419 0.0.75 Luc EE0704005: Light blue wash
#   070521 0.0.76 Luc EE0704005: Copy CKFREEZE to IOCONFIG properties
###########################################################EndFile##############

# $1 = Configuration file
# $2 = Collect strategy
# $3 = Store raw files
# $4 = Error retention
# $5 = Complex name
# $6 = CKFREEZE Data Set Name
# $7 = UNLOAD Data Set Name
# $8 = System Policy Type
# $9 = State Prefix

early_terminate ()
{
    [[ "$RUN_SUBDIR" == "" ]] || rm -rf $RUN_SUBDIR
    [[ -n "$3" ]] && echo "`date`: `basename $0`: $PORT: $3"
    [[ -n "$3" ]] && logger -a "C2E$2I: $PORT" "$3"
    logger -a "C2E202I: $PORT" "End of `basename $0` with rc=$1; id=$ID"
    [[ $1 -ne 0 ]] && logger -a "C2E203I: $PORT" "Agent root is `dirname $PWD`"
    exit $1
}

store_properties ()
{
    # Store properties and last (successful) collect dates
    echo "Configuration file=$GROUP_CFG_FILE" > $PARAMETER_FILE
    echo "Collect strategy=$STRATEGY" >> $PARAMETER_FILE
    echo "Store raw files=$RAW" >> $PARAMETER_FILE
    echo "Last collect=$LAST_COLLECT_DATE" >> $PARAMETER_FILE
    echo "Last successful collect=$LAST_SUCCESS_DATE" >> $PARAMETER_FILE
    echo "Error retention=$RETPD" >> $PARAMETER_FILE
    echo "Complex name=$COMPLEX" >> $PARAMETER_FILE
    echo "CKFREEZE Data Set Name=$CKFREEZE" >> $PARAMETER_FILE
    echo "IOCONFIG Data Set Name=$CKFREEZE" >> $PARAMETER_FILE     # enable reversion to software which expects IOCONFIG
    echo "UNLOAD Data Set Name=$UNLOAD" >> $PARAMETER_FILE
    echo "System Policy Type=$POLICY_TYPE" >> $PARAMETER_FILE
}

refresh_master ()
{
    # Refresh master data set (including information just written to $PARAMETER_FILE)
    echo "[Agent]" > $MASTER_CONTENTS
    # Determine agent ID
    # This assumes ../bin/db2asc ../run/startup has a single line which contains " 1.2.3.15(int)" and which looks like ... <id>(..., e.g.
    # ((objid) 1.2.3.15(int) 1) = (((uint) 80(objval) ((objid) 12.1.136(objid) 1.2.3.14(objval) ((objval) ((uint) 1(uint) 4)))))
    echo "Id=`../bin/db2asc ../run/startup | grep ' 1.2.3.15(int)' | awk -F\  '{print $8}' | awk -F\( '{print $1}'`" >> $MASTER_CONTENTS
    # Determine server name (or IP) and port
    # This assumes ../bin/db2asc ../run/startup has a single line which contains ""A" and which looks like ...:<name or IP>:<port number>"..., e.g.
    # ((objid) 1.2.3.4(int) 3) = (((uint) 1920(objval) ((int) 1(string) "12.1.1"(string) "0"(string) "A:CRMNLW016:5050"(objval) ())))
    SERVER=`../bin/db2asc ../run/startup | awk -F: '/\"A/ {print $2 "\"" $3}' | awk -F\" '{print $1 ":" $2}'`
    echo "Server=$SERVER" >> $MASTER_CONTENTS
    CFG=""
    [[ -f ../run/configuration_file_name ]] && CFG=`cat ../run/configuration_file_name`
    echo "ConfigFile=$CFG" >> $MASTER_CONTENTS
    for ES_STATE in `ls $ES_STATES | grep -v cutoff`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[EventSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $ES_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS             # ES_STATE = <x>.<y>.<z>.props
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "SMF=`awk -F= '/SMF Data Set Name/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
      echo "Collect_Past_Data=`awk -F= '/Collect past data \(discouraged\)/ {print $2}' $ES_STATES/$ES_STATE`" >> $MASTER_CONTENTS
    done
    for UIS_STATE in `ls $UIS_STATES`
    do
      echo "" >> $MASTER_CONTENTS
      echo "[UserInformationSource]" >> $MASTER_CONTENTS
      echo "Id=`echo $UIS_STATE | awk -F. '{print $1 "." $2 "." $3}'`" >> $MASTER_CONTENTS        # UIS_STATE = <x>.<y>.<z>.props
      echo "Complex=`awk -F= '/Complex name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Strategy=`awk -F= '/Collect strategy/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "CKFREEZE=`awk -F= '/CKFREEZE Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "UNLOAD=`awk -F= '/UNLOAD Data Set Name/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Collect=`awk -F= '/Last collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Last_Successful_Collect=`awk -F= '/Last successful collect/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
      echo "Error_Retention=`awk -F= '/Error retention/ {print $2}' $UIS_STATES/$UIS_STATE`" >> $MASTER_CONTENTS
    done
    cp $MASTER_CONTENTS "//'$MASTER'"
}

# Determine agent port number
# This assumes ../bin/db2asc ../run/startup has a single line which contains "listen" and which looks like ...:...:<port number>"..., e.g.
# ((string) "listen") = (((string) "0"(string) "P:DEST_ANY:7300"(objval) ()))
PORT=`../bin/db2asc ../run/startup | awk -F: '/\"listen\"/ {print $3}' | awk -F\" '{print $1}'`

# Check if this script is executed within the correct working directory.
[[ `basename $PWD` = "run" && -d ../bin && -d ../log ]] || early_terminate 8 204 "$0 must be executed from the run directory"

# Check number of arguments
[[ $# -ge 9 ]] || early_terminate 8 205 "$0 expects at least 9 arguments"

# Determine output and log directory prefix
STATE_PREFIX=$9
ID=`echo $STATE_PREFIX | awk -F. '{print $1 "." $2 "." $3}'` # <x>.<y>.<z>.props

# Log script start to console
logger -a "C2E201I: $PORT" "Start of `basename $0`; id=$ID"
logger -a "C2E203I: $PORT" "Agent root is `dirname $PWD`"

# Determine user information source and event source state directories
UIS_STATES="../run/UIS_properties"
ES_STATES="../run/ES_properties"

# Determine user information source parameter file
PARAMETER_FILE="$UIS_STATES/$STATE_PREFIX"

# Determine file name components
RUN_SUBDIR="$PWD/../run/$STATE_PREFIX.tmp"

# Create directories if they are not yet there.
[[ -d $RUN_SUBDIR ]] || mkdir -p $RUN_SUBDIR
[[ -d $ES_STATES ]] || mkdir $ES_STATES
[[ -d $UIS_STATES ]] || mkdir $UIS_STATES

# Set directory permission bits
getfacl ../run | setfacl -S - $RUN_SUBDIR
getfacl ../run | setfacl -S - $UIS_STATES
getfacl ../run | setfacl -S - $ES_STATES

if [ -f CKR ]
then
  TCKFREEZE="CKFREEZE"
else
  TCKFREEZE="IOCONFIG"
fi

# Determine file names
MASTER_CONTENTS="$RUN_SUBDIR/master_contents"               # tmp

# Determine MASTER and ENQ data set names, as well as some of their qualifiers
[[ -s ../run/master ]] || terminate 8 206 "The master file in the run directory does not exist or is empty"
MASTER=`cat ../run/master`
C2ELVPFX=`echo $MASTER | awk -F. '{prefix = ""}{for(i=1; i<=NF && $i != "MASTER"; i++) {prefix = prefix $i "."}}{print prefix}'`
MASTERQUAL=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)} {if(i <= NF) {print "MASTER"}}'`
[[ "$MASTERQUAL" != "" ]] || terminate 8 207 "The master data set specified in the master file in the run directory should have a MASTER qualifier"
C2ELVLLQ=`echo $MASTER | awk -F. '{for(i=1; i<=NF && $i != "MASTER"; i++)}{suffix = ""}{for(j=i+1; j<=NF; j++){suffix = suffix "." $j}}{print suffix}'`
ENQ="${C2ELVPFX}ENQ$C2ELVLLQ"
DEFAULT_CKFREEZE="${C2ELVPFX}${TCKFREEZE}$C2ELVLLQ"

# Read properties
GROUP_CFG_FILE=$1
STRATEGY=$2
RAW=$3
RETPD=$4
COMPLEX=$5
CKFREEZE=$6
UNLOAD=$7
POLICY_TYPE=$8

# Convert some properties to uppercase
STRATEGY=`echo $STRATEGY | tr '[:lower:]' '[:upper:]'`
RAW=`echo $RAW | tr '[:lower:]' '[:upper:]'`
COMPLEX=`echo $COMPLEX | tr '[:lower:]' '[:upper:]'`
CKFREEZE=`echo $CKFREEZE | tr '[:lower:]' '[:upper:]'`
UNLOAD=`echo $UNLOAD | tr '[:lower:]' '[:upper:]'`
POLICY_TYPE=`echo $POLICY_TYPE | tr '[:lower:]' '[:upper:]'`
# Convert POLICY_TYPE to Production or Test
[[ "$POLICY_TYPE" == "PRODUCTION" ]] && POLICY_TYPE="Production"
[[ "$POLICY_TYPE" == "TEST" ]] && POLICY_TYPE="Test"

[[ "$CKFREEZE" == "" ]] && CKFREEZE=$DEFAULT_CKFREEZE

# Store properties
touch $PARAMETER_FILE  # make sure the next two commands do not fail
LAST_COLLECT_DATE=`awk -F= '/Last collect/ {print $2}' $PARAMETER_FILE`
LAST_SUCCESS_DATE=`awk -F= '/Last successful collect/ {print $2}' $PARAMETER_FILE`

store_properties
refresh_master

early_terminate 0
